%=====clean up any leftover serial connections==============
clear all
try
    fclose(instrfind) %close any bogus serial connections
end

%=====open a serial connection===============================
%set its rate to 9600 baud
%SR830 terminator character is  (ACSII 13)
%use fprintf(s,['m' mode]) to write
%find which COM port you need!!!!
s = serial('COM1','baudrate',9600,'terminator',13); 
fopen(s);
disp('serial communication setup complete');

%defines and init values
PLOT_SIZE = 200;
CALIB_CONST = 20;
vect = zeros(1, 200);
figure(1);
pulse = plot(vect);
pulseTitle = title('Freq ~ BPM ~');

%infinite loop to continously poll serial comm and plot data
while (1)     
    vect = vect([ 2:end 1 ]); %left shift data vector
    temp = fscanf(s, '%d'); %get data from serial
    if (temp <= 255) %only use values that aren't bogus, ADC values range from 0-255
        vect(PLOT_SIZE) = 5*temp/255; %adjust ADC value from 0-5V
        
        thres = (min(vect)+max(vect))/2; %calculate threshold
        counti = 0;
        for i=1:200 %count number of pulses that pass threshold in data vector
           if (i > 1)
               if (vect(i) >= thres && vect(i-1) <= thres)
                   counti = counti + 1;
               end               
           end
        end
        set(pulse, 'YData', vect); %update the plot with new data
        drawnow;
        pfreq = CALIB_CONST*counti/PLOT_SIZE; %get frequency from count/total size adjusted by constant
        pbpm = 60*pfreq;
        pTitle = sprintf('Freq ~ %5.2f Hz BPM ~ %d', pfreq, pbpm); %update frequency value and BPM value
        set(pulseTitle, 'String', pTitle);        
    end
end