// INTERRUPT-DRIVEN serial code 
//used as an example in the serial communication doc.      
             
#include <mega32.h> 
#include <stdio.h>       
#include <delay.h>

//timeout values for each task
#define t1 100  
#define t2 1000  

//the  subroutines
void task1(void);  		//test for button press
void task2(void);		//increment note to be played  
void gets_int(void);	//starts getting a string from serial line
void puts_int(void);	//starts a send to serial line
void initialize(void); 	//all the usual mcu stuff 
          
unsigned int time1, time2	;	//task scheduling timeout counters
unsigned long time;
unsigned int v;
	
//RXC ISR variables								
unsigned char r_index;  //current string index
unsigned char r_buffer[16];	//input string 
unsigned char r_ready;  //flag for receive done
unsigned char r_char;  //current character  

//TX empth ISR variables								
unsigned char t_index;  //current string index
unsigned char t_buffer[16];	//output string 
unsigned char t_ready;  //flag for transmit done
unsigned char t_char;  //current character
   
//**********************************************************
//timer 0 compare-match ISR, every 1 msec
interrupt [TIM0_COMP] void timer0_overflow(void)
{   
   time++;
  //Decrement the three times if they are not already zero
  if (time1>0)	--time1;
  if (time2>0) 	--time2;
}

//**********************************************************
//UART character-ready ISR
interrupt [USART_RXC] void uart_rec(void)
{
	r_char=UDR;    //get a char
	UDR=r_char;    //then print it
	//build the input string
	if (r_char != '\r') r_buffer[r_index++]=r_char;
	else
	{
		putchar('\n');  			//use putchar to avoid overwrite
		r_buffer[r_index]=0x00;    	//zero terminate
		r_ready=1;				   	//signal cmd processor
		UCSRB.7=0;   				//stop rec ISR
	}
}

/**********************************************************/
//UART xmit-empty ISR
interrupt [USART_DRE] void uart_send(void)
{
	t_char = t_buffer[++t_index];
	if (t_char == 0) 	
	{
		UCSRB.5=0; //kill isr 
		t_ready=1; //transmit done 
	}
	else	UDR = t_char ;     //send the char 
}
/**********************************************************/ 
//**********************************************************       
//Entry point and task scheduler loop
void main(void)
{  
  initialize();
  
  //main task scheduler loop -- never exits!
  while(1)
  {                                      
    ADCSRA.6 = 0x1;             //start the conversion
  	while (ADCSRA.6 == 0x1) {}  //wait until conversion finishes

	printf("%d\r\n", ADCH);     //print ADC value to serial
    puts_int();
   	delay_ms(20);               //get next ADC conversion after 20 ms for timing issues
    if (time1==0) task1();
//    if (time2==0) task2();
  }   
} 
  
//**********************************************************          
//Task 1   input a string and print it
void task1(void) 
{

  time1=t1;  				//reset the task timer 
  //print and get another serial string      
  if (r_ready )    
  {
  	//sscanf(r_buffer,"%d",&v);
    //gets_int();     
  }     
}  
 
//********************************************************** 
//Task 2  print the system time
void task2(void) 
{
  time2=t2;  				//reset the task timer
  //sprintf(t_buffer,"%ld %d\n\r",time,v) ;  
  puts_int(); 
}  
  
//********************************************************** 
//  -- non-blocking keyboard check initializes ISR-driven
// receive. This routine merely sets up the ISR, which then
//does all the work of getting a command.
void gets_int(void) 
{
  r_ready=0;
  r_index=0;
  UCSRB.7=1;  // RX Complete Interrupt Enable
}
      
//********************************************************** 
//  -- nonblocking print: initializes ISR-driven
// transmit. This routine merely sets up the ISR, then
//send one character, The ISR does all the work.   
void puts_int(void) 
{
  t_ready=0;
  t_index=0;
  if (t_buffer[0]>0) 
  {
  	putchar(t_buffer[0]);
  	UCSRB.5=1; // USART Data Register Empty Interrupt Enable
  }
}     

//********************************************************** 
//Set it all up
void initialize(void)
{
  //serial setop for debugging using printf, etc.     
  UCSRB = 0x18 ; // Tx and Rx enable
  UBRRL = 103 ;  // set baud rate to 9600 for single rate transmission and 16MHz crystal
         
  //set up timer 0
  TCCR0=0b00001011;		// prescalar to 64  and Clr-on-match 
  OCR0=249;	   			// 16MHz/64 prescalar * 250 counts -> set output compare register to 1 mSec
  TIMSK=2;	   			// turn on timer 0 output compare-match interrupt
  
  //init the task timers
  time1=t1;
  time2=t2;    
  
  r_ready=0;
  t_ready=1;

  ADMUX = 0b01100000;  //ref = AVCC, ADC0
  ADCSRA = 0b11000111; //prescaler = 128 => ADC speed 16MHz/128 = 128 kHz

  ADCSRA.6 = 0x1;            //do an ADC conversion to get out any bugs	
  while (ADCSRA.6 == 0b0) {} 
	     
  //crank up the ISRs
  #asm
  	sei
  #endasm  
 
  gets_int();
}
